/**
 *****************************************************************************
   @example  ADT7310I2C.c
   @brief    Set of functions to interface ADuCM360/61 to ADT7310 via SPI.
	           The SPI peripheral shoud be initialised before using these functions.

   @version  V0.1
   @author   ADI
   @date     July 2013 
              

All files for ADuCM360/361 provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <ADuCM360.h>
#include <..\common\SpiLib.h>
#include "ADT7310SPI.h"

unsigned char ucTxComplete = 0;
unsigned char SPIRxData[4];
unsigned char ResolutionSetting;
unsigned loading = 0;
unsigned short temp    = 0;
unsigned int install_cont_read = 0;


/** 
   @fn     ADT7310_SPIGetTemperature(void)
   @brief  Return temperature measurement in degree Celsius.
   @pre    ADT7310_SPICFG() must be called before this function is called.
   @return float Temperature measurement
**/
float ADT7310_SPIGetTemperature(void)
{  
  unsigned char msbTemp = 0;
  unsigned char lsbTemp = 0; 
  float SPItempC;

  SpiTx(pADI_SPI0,0x50);
  SpiTx(pADI_SPI0,0xFF);
  while (ucTxComplete == 0) {}
  ucTxComplete = 0;
  SpiTx(pADI_SPI0,0xFF);
  SpiTx(pADI_SPI0,0xFF);
  while (ucTxComplete == 0) {}
  ucTxComplete = 0;

  msbTemp = SPIRxData[1];
  lsbTemp = SPIRxData[2];
    
  temp = ((unsigned short)msbTemp << 8) + lsbTemp;
  if(ResolutionSetting)
  {
    if(temp & 0x8000)           // Negative temperature 
    {
      SPItempC = (float)((signed long)temp - 65536) / 128;
    }
    else                       // Positive temperature 
    {
      SPItempC = (float)temp / 128;
    }
  }
  else
  {
    temp >>= 3;
    if(temp & 0x1000)        // Negative temperature 
    {
      SPItempC = (float)((signed long)temp - 8192) / 16;
    }
    else                     // Positive temperature
    {
      SPItempC = (float)temp / 16;
    }
  }
  return SPItempC;
}

/** 
   @fn ADT7310_SPICFG(int Resolution,int iMode,int CT,int INT,int INTCTmode,int Fault_queue)
   @brief Configure the temperature sensor resolution, mode, CT pin polarity, INT pin polarity etc
   @param Resolution :{Bit_16, Bit_13} 
         - select the resolution of the ADC:
         - Bit_16 for 16 bit resolution
         - Bit_13 for 13 bit resolution
   @param iMode :{cont_conv, one_shot, SPS_mode, Shutdown} 
         - select the operational mode of the ADT7310:
         - cont_conv for continuous mode
         - one_shot for one shot mode
         - SPS_mode for 1SPS mode
         - Shutdown for shutdown mode
   @param CT :{CT_pin_L, CT_pin_H} 
         - select the output polarity of the CT pin:
         - CT_pin_L for active low
         - CT_pin_H for active high
   @param INT :{INT_pin_L, INT_pin_H} 
         - select the output polarity of the INT pin:
         - INT_pin_L for active low
         - INT_pin_H for active high
   @param INTCTmode :{interupt_mode, comparator_mode} 
         - selects between comparator mode and interrupt mode: 
         - interupt_mode
         - comparator_mode 
   @param Fault_queue :{fault_1, fault_2, fault_3, fault_4}
         - set the number of overtemperature/undertemperature faults that can occur before setting the INT and CT pins.
         - fault_1     
         - fault_2     
         - fault_3     
         - fault_4     
   @pre     SPI must be configured before this function is called.
   @return  1
**/
int ADT7310_SPICFG(int Resolution,int iMode,int CT,int INT,int INTCTmode,int Fault_queue)
{
  int il;
  il = Fault_queue << 1;
  il |= CT << 2;
  il |= INT << 3;
  il |= INTCTmode << 4;
  il |= (iMode&2) << 5;
  il |= Resolution << 7;
  ResolutionSetting = Resolution;
  SpiTx(pADI_SPI0,0x08); // write to register address 0x01
  SpiTx(pADI_SPI0,il);
  while (ucTxComplete == 0) {}
  ucTxComplete = 0;
  SpiTx(pADI_SPI0,0xFF);
  SpiTx(pADI_SPI0,0xFF);
  while (ucTxComplete == 0) {}
  ucTxComplete = 0;	
  return 1;
}

/** 
   @fn ADT7310_SPI_Status(char REG_Address)
   @brief Return content of the register at REG_Address (status, configuration, ID or T_setpoint)
   @param REG_Address :{Status, Configuration, REG_ID, T_Crit_Setpoint, T_Hyst_Setpoint, T_High_Setpoint, T_Low_Setpoint} 
         - select the register to read:
         - Status
         - Configuration
         - REG_ID
         - T_Crit_Setpoint
         - T_Hyst_Setpoint
         - T_High_Setpoint 
         - T_Low_Setpoint      
   @return content of the register at REG_Address
**/
long ADT7310_SPI_Status(char REG_Address)
{
  int il = 0;
  unsigned long status;

  il = (1 << 6) + (REG_Address << 3);  // SPI command byte
  SpiTx(pADI_SPI0,il);
  SpiTx(pADI_SPI0,0xFF);
  while (ucTxComplete == 0){}
  ucTxComplete = 0;
  SpiTx(pADI_SPI0,0xFF);
  SpiTx(pADI_SPI0,0xFF);
  while (ucTxComplete == 0) {}
  ucTxComplete = 0;
      
  if(REG_Address == REG_ID ||REG_Address == Status || REG_Address == Configuration || REG_Address == T_Hyst_Setpoint)
  {
    status = SPIRxData[1];
    return status;
  }
  else
  {
    status = (SPIRxData[1] << 8);
    status = status + SPIRxData[2];
    return status;
  }
}

/** 
   @fn ADT7310_SPI_T_Setpoint(int REG_Address,int value)
   @brief Configure one of the four set point register (THIGH,TLOW,TCRIT or THYST)
   @param REG_Address :{T_Crit_Setpoint, T_Hyst_Setpoint, T_High_Setpoint, T_Low_Setpoint} 
         - select the register to write:
         - T_Crit_Setpoint
         - T_Hyst_Setpoint
         - T_High_Setpoint 
         - T_Low_Setpoint  
   @param value :{0-256}     
   @return 1
**/
int ADT7310_SPI_T_Setpoint(int REG_Address,int value)
{
  int il = 0;
	
  il = (REG_Address << 3);  // SPI command byte
  if(REG_Address == T_Hyst_Setpoint)
  {
    SpiTx(pADI_SPI0,il);
    SpiTx(pADI_SPI0,value);
    while (ucTxComplete == 0) {}
    ucTxComplete = 0;
    SpiTx(pADI_SPI0,0xFF);
    SpiTx(pADI_SPI0,0xFF);
    while (ucTxComplete == 0) {}
    ucTxComplete = 0;	
  }
  else
  {
    SpiTx(pADI_SPI0,il);
    SpiTx(pADI_SPI0,(value >> 8));
    while (ucTxComplete == 0) {}
    ucTxComplete = 0;
    SpiTx(pADI_SPI0,(value & 0xFF));
    SpiTx(pADI_SPI0,0xFF);
    while (ucTxComplete == 0) {}
    ucTxComplete = 0;
  }
  return 1;
}

/** 
   @fn ADT7310_Reset(void)
   @brief Reset the ADT7310
   @return 1
**/
int ADT7310_Reset(void)
{
  unsigned int delay = 0;

  SpiTx(pADI_SPI0,0xFF);
  SpiTx(pADI_SPI0,0xFF);
  SpiTx(pADI_SPI0,0xFF);
  SpiTx(pADI_SPI0,0xFF);
  SpiTx(pADI_SPI0,0xFF);
  SpiTx(pADI_SPI0,0xFF);
  SpiTx(pADI_SPI0,0xFF);
  SpiTx(pADI_SPI0,0xFF);
  while(delay <= 0x5FFFF)
    delay++;
  return 1;
}



void SPI0_Int_Handler ()
{
  unsigned int uiSPI0STA = 0;
  unsigned int uiSPI0STA1 = 0;

  uiSPI0STA = SpiSta(pADI_SPI0);
  if ((uiSPI0STA & SPI0STA_RXOF) == SPI0STA_RXOF)// SPI0 Rx Overflow IRQ
  {
  }
  if ((uiSPI0STA & SPI0STA_RX) == SPI0STA_RX)	 	 // SPI0 Rx IRQ
  {

  }
  if ((uiSPI0STA & SPI0STA_TX) == SPI0STA_TX)	 	 // SPI0 Tx IRQ
  {
    do
    {
      uiSPI0STA1 = pADI_SPI0->SPISTA & SPI0STA_RXFSTA_MSK;
    }
    while(uiSPI0STA1 != SPI0STA_RXFSTA_TWOBYTES);
    SPIRxData[loading++] = SpiRx(pADI_SPI0);
    SPIRxData[loading++] = SpiRx(pADI_SPI0);
    ucTxComplete = 1;
    if(loading >= 4) loading = 0;
  }
  if ((uiSPI0STA & SPI0STA_TXUR) == SPI0STA_TXUR)// SPI0 Tx underflow IRQ
  {
  }
  if((uiSPI0STA & SPI0STA_TXFSTA_TWOBYTES) == SPI0STA_TXFSTA_TWOBYTES)
  {
  }
}

